@@ -1,18 +1,19 @@
 /***************************************************************************
  * LPRng - An Extended Print Spooler System
  *
- * Copyright 1988-2002, Patrick Powell, San Diego, CA
+ * Copyright 1988-2003, Patrick Powell, San Diego, CA
  *     papowell@lprng.com
  * See LICENSE for conditions of use.
  *
  ***************************************************************************/
 
  static char *const _id =
-"$Id: checkpc.c,v 1.27 2002/04/01 17:54:50 papowell Exp $";
+"$Id: checkpc.c,v 1.71 2004/05/03 20:24:01 papowell Exp $";
 
 
 
 #include "lp.h"
+#include "defs.h"
 #include "getopt.h"
 #include "checkpc.h"
 #include "patchlevel.h"
@@ -50,7 +51,7 @@ int main( int argc, char *argv[], char *envp[] )
 	char *path;		/* end of string */
 	int ruid, euid, rgid, egid;
 	char *printcap;
-	char *serial_line = 0;
+	/*char *serial_line = 0;*/
 	struct line_list raw, spooldirs;
 	char *s, *t;
 	struct stat statb;
@@ -113,7 +114,7 @@ int main( int argc, char *argv[], char *envp[] )
 			case 'P': User_specified_printer = Optarg; break;
 			case 'T':
 				initsetproctitle( argc, argv, envp );
-				Test_port( getuid(), geteuid(), serial_line );
+				Test_port( getuid(), geteuid(), Optarg );
 				exit(0);
 				break;
 		}
@@ -139,7 +140,11 @@ int main( int argc, char *argv[], char *envp[] )
 		&PC_info_line_list, &raw, &Host_IP );
 	Free_line_list( &raw );
 
+#if defined (__CYGWIN__)
+	if( Fix && (geteuid() != ROOTUID && getuid() != ROOTUID) ){
+#else
 	if( Fix && geteuid() && getuid() ){
+#endif
 		WARNMSG("Fix option (-f) requires root permissions\n" );
 	}
 
@@ -217,7 +222,7 @@ int main( int argc, char *argv[], char *envp[] )
 	if(Verbose)MESSAGE("Checking printcap info");
 	if( User_specified_printer ){
 		if( DEBUGL1 ) Dump_line_list("checkpc: names", &PC_names_line_list );
-		s = Find_str_value( &PC_names_line_list, User_specified_printer, Value_sep );
+		s = Find_str_value( &PC_names_line_list, User_specified_printer );
 		DEBUG1("checkpc: for SERVER %s is really %s", User_specified_printer, s );
 		if( s ){
 			Set_DYN(&Printer_DYN,s);
@@ -291,12 +296,13 @@ void Scan_printer(struct line_list *spooldirs)
 	int jobfile;
 	struct stat statb;
 	int fd = 0;				/* device file descriptor */
-	int i, n;
+	int i, n, fifo_header_len;
 	char error[SMALLBUFFER];
 	int errorlen = sizeof(error);
 	struct job job;
 	time_t delta;
 
+	fifo_header_len = safestrlen( Fifo_lock_file_DYN );
 	Init_job(&job);
 	error[0] = 0;
 
@@ -329,20 +335,17 @@ void Scan_printer(struct line_list *spooldirs)
 		}
 		goto test_filters;
 	}
-	if( Is_server && Client_flag_DYN ){
-		return;
-	}
-	if( !Find_first_key(&PC_entry_line_list,"bq",Value_sep,&n)
-		|| !Find_first_key(&Config_line_list,"bq",Value_sep,&n ) ){
+	if( !Find_first_key(&PC_entry_line_list,"bq",Option_value_sep,&n)
+		|| !Find_first_key(&Config_line_list,"bq",Option_value_sep,&n ) ){
 		WARNMSG( "%s: bq option is no longer supported, use 'lpd_bounce' option", Printer_DYN);
 	}
-	if( !Find_first_key(&PC_entry_line_list,"check_idle",Value_sep,&n)
-		|| !Find_first_key(&Config_line_list,"check_idle",Value_sep,&n ) ){
+	if( !Find_first_key(&PC_entry_line_list,"check_idle",Option_value_sep,&n)
+		|| !Find_first_key(&Config_line_list,"check_idle",Option_value_sep,&n ) ){
 		WARNMSG( "%s: check_idle option is no longer supported, use 'chooser' option", Printer_DYN);
 	}
-	if( !Find_first_key(&PC_entry_line_list,"sf",Value_sep,&n)
-		|| !Find_first_key(&Config_line_list,"sf",Value_sep,&n ) ){
-		WARNMSG( "%s: sf (suppress form feeds) is ignored. Use 'ff_separator' if you want FF between job files", Printer_DYN);
+	if( !Find_first_key(&PC_entry_line_list,"sf",Option_value_sep,&n)
+		|| !Find_first_key(&Config_line_list,"sf",Option_value_sep,&n ) ){
+		WARNMSG( "%s: sf (suppress form feeds) is deprecated.  Use 'ff_separator' if you want FF between job files", Printer_DYN);
 	}
 	if( strchr(Printer_DYN, '*') ){
 		WARNMSG(
@@ -361,8 +364,8 @@ void Scan_printer(struct line_list *spooldirs)
 			Printer_DYN);
 		return;
 	}
-	if( (s =  Find_str_value(spooldirs,Spool_dir_DYN,Value_sep)) ){
-		WARNMSG("%s: CATASTOPHIC ERROR! queue '%s' also has spool directory '%s'",
+	if( (s =  Find_str_value(spooldirs,Spool_dir_DYN)) ){
+		WARNMSG("%s: CATASTROPHIC ERROR! queue '%s' also has spool directory '%s'",
 			Printer_DYN, s, Spool_dir_DYN);
 		return;
 	}
@@ -391,6 +394,12 @@ void Scan_printer(struct line_list *spooldirs)
 		if( safestrcmp( cf_name, "." ) == 0
 			|| safestrcmp( cf_name, ".." ) == 0 ) continue;
 		DEBUG2("Scan_printer: file '%s'", cf_name );
+		if( fifo_header_len &&
+			!safestrncmp( cf_name,Fifo_lock_file_DYN, fifo_header_len) ){
+			DEBUG2("Scan_printer: fifo file '%s'", cf_name );
+			unlink( cf_name );
+			continue;
+		}
 		if( stat(cf_name,&statb) == -1 ){
 			WARNMSG( "  stat of file '%s' failed '%s'",
 				cf_name, Errormsg(errno) );
@@ -456,7 +465,9 @@ void Scan_printer(struct line_list *spooldirs)
 	Fix_clean(Status_file_DYN,Nostatus);
 	Fix_clean(Log_file_DYN,Nolog);
 	Fix_clean(Accounting_file_DYN,Noaccount);
-
+	if( (s = Ppd_file_DYN) ){
+		Check_read_file( s, Fix, 0644, 0 );
+	}
 	/*
 	 * get the jobs in the queue
 	 */
@@ -465,7 +476,7 @@ void Scan_printer(struct line_list *spooldirs)
 	}
 	Free_line_list( &Sort_order );
 	{ int fdx = open("/dev/null",O_RDWR); DEBUG1("Scan_printer: Scan_queue before maxfd %d", fdx); close(fdx); }
-	Scan_queue( &Spool_control, &Sort_order,0,0,0,0, 0, 0 );
+	Scan_queue( &Spool_control, &Sort_order,0,0,0,0, 0, 0,0,0 );
 	{ int fdx = open("/dev/null",O_RDWR); DEBUG1("Scan_printer: Scan_queue after maxfd %d", fdx); close(fdx); }
 
 	/*
@@ -529,8 +540,8 @@ void Check_executable_filter( char *id, char *filter_str )
 	
 	Init_line_list(&files);
 	if( !filter_str ){
-		filter_str = Find_str_value(&PC_entry_line_list,id,Value_sep);
-		if(!filter_str) filter_str = Find_str_value(&Config_line_list,id,Value_sep);
+		filter_str = Find_str_value(&PC_entry_line_list,id);
+		if(!filter_str) filter_str = Find_str_value(&Config_line_list,id);
 	}
 	Split(&files,filter_str,Whitespace,0,0,0,0,0,0);
 	if( files.count ){
@@ -713,15 +724,15 @@ int Check_file( char  *path, int fix, int age, int rmflag )
 		path, fix, (long)Current_time, age );
 
 	if( stat( path, &statb ) ){
-		if(Verbose)MESSAGE( "cannot stat file '%s', %s", path, Errormsg(errno) );
+		WARNMSG( "  %s: cannot stat file '%s', %s", Printer_DYN?Printer_DYN:"", path, Errormsg(errno) );
 		err = 1;
 		return( err );
 	}
 	if( S_ISDIR( statb.st_mode ) ){
-		WARNMSG("'%s' is a directory, not a file", path );
+		WARNMSG("  %s: '%s' is a directory, not a file", Printer_DYN?Printer_DYN:"",path );
 		return(2);
 	} else if( !S_ISREG( statb.st_mode ) ){
-		WARNMSG( "'%s' not a regular file - unusual", path );
+		WARNMSG( " %s: '%s' not a regular file - unusual", Printer_DYN?Printer_DYN:"",path );
 		return(2) ;
 	}
 
@@ -757,6 +768,47 @@ int Check_file( char  *path, int fix, int age, int rmflag )
 	return( err );
 }
 
+
+/***************************************************************************
+ * Check_read_file( char  *dpath   - pathname of directory/files
+ *    int fix  - fix or check
+ ***************************************************************************/
+
+int Check_read_file( char  *path, int fix, int perms, int owner )
+{
+	struct stat statb;
+	int err = 0;
+	int fd;
+
+	DEBUG4("Check_read_file: '%s', fix %d", path, fix );
+
+	if( stat( path, &statb ) ){
+		WARNMSG( "  %s: cannot stat file '%s', %s", Printer_DYN?Printer_DYN:"",
+			path, Errormsg(errno) );
+		err = 1;
+		return( err );
+	}
+	if( S_ISDIR( statb.st_mode ) ){
+		WARNMSG("  %s: '%s' is a directory, not a file", Printer_DYN?Printer_DYN:"",path );
+		return(2);
+	} else if( !S_ISREG( statb.st_mode ) ){
+		WARNMSG( " %s: '%s' not a regular file - unusual", Printer_DYN?Printer_DYN:"",path );
+		return(2) ;
+	}
+	if( (fd = Checkread( path, &statb )) < 0 ){
+		if( fix ){
+			if( owner ) Fix_owner( path );
+			Fix_perms( path, perms );
+		} else {
+			WARNMSG( " %s: cannot open %s - %s", Printer_DYN?Printer_DYN:"",
+				path, Errormsg(errno) );
+		}
+	}
+	if( fd >= 0 ) close(fd);
+
+	return( err );
+}
+
 int Fix_create_dir( char  *path, struct stat *statb )
 {
 	char *s;
@@ -961,7 +1013,7 @@ void Test_port(int ruid, int euid, char *serial_line )
 	static int fd;
 	static int i, err;
 	struct stat statb;
-	char *Stty_command;
+	/*char *Stty_command;*/
 
 	status = 0;
 	fd = -1;
@@ -1195,6 +1247,9 @@ void Test_port(int ruid, int euid, char *serial_line )
 			sttycmd = "stty -a >%s";	/* on STDOUT */
 #elif defined(BSD) /* old style BSD4.[23] */
 			sttycmd = "stty everything 2>%s";
+#else /* That is: All other System V derivatives and AIX as well */
+			ttyfd = 0;	/* STDIN is reported */
+			sttycmd = "/bin/stty -a >%s";	/* on STDOUT */
 #endif
 			if( fd != ttyfd ){
 				i = dup2(fd, ttyfd );
@@ -1212,8 +1267,8 @@ void Test_port(int ruid, int euid, char *serial_line )
 				cmd, fd, ttyfd );
 			i = system( cmd );
 			FPRINTF( STDERR, "\n\n" );
-			Stty_command = "9600 -even odd echo";
-			FPRINTF( STDERR, "Trying 'stty %s'\n", Stty_command );
+			Stty_command_DYN = "9600 -even odd echo";
+			FPRINTF( STDERR, "Trying 'stty %s'\n", Stty_command_DYN );
 			Do_stty( ttyfd );
 			SNPRINTF( stty, sizeof(stty)) sttycmd, t2 );
 			SNPRINTF( cmd, sizeof(cmd))
@@ -1221,14 +1276,14 @@ void Test_port(int ruid, int euid, char *serial_line )
 			FPRINTF( STDERR, "Doing '%s'\n", cmd );
 			i = system( cmd );
 			FPRINTF( STDERR, "\n\n" );
-			Stty_command = "1200 -odd even";
-			FPRINTF( STDERR, "Trying 'stty %s'\n", Stty_command );
+			Stty_command_DYN = "1200 -odd even";
+			FPRINTF( STDERR, "Trying 'stty %s'\n", Stty_command_DYN );
 			Do_stty( ttyfd );
 			FPRINTF( STDERR, "Doing '%s'\n", cmd );
 			i = system( cmd );
 			FPRINTF( STDERR, "\n\n" );
-			Stty_command = "300 -even -odd -echo cbreak";
-			FPRINTF( STDERR, "Trying 'stty %s'\n", Stty_command );
+			Stty_command_DYN = "300 -even -odd -echo cbreak";
+			FPRINTF( STDERR, "Trying 'stty %s'\n", Stty_command_DYN );
 			Do_stty( ttyfd );
 			SNPRINTF( stty, sizeof(stty)) sttycmd, serial_line, t2 );
 			FPRINTF( STDERR, "Doing '%s'\n", cmd );
@@ -1355,7 +1410,7 @@ void Test_port(int ruid, int euid, char *serial_line )
 		exit(0);
 	}
 	FPRINTF( STDERR, "Mother pausing before releasing lock on fd %d\n", fd );
-	sleep(3);
+	plp_sleep(3);
 
 	FPRINTF( STDERR, "Mother closing '%s', releasing lock on fd %d\n",
 		line, fd );
