@@ -1,14 +1,14 @@
 /***************************************************************************
  * LPRng - An Extended Print Spooler System
  *
- * Copyright 1988-2002, Patrick Powell, San Diego, CA
+ * Copyright 1988-2003, Patrick Powell, San Diego, CA
  *     papowell@lprng.com
  * See LICENSE for conditions of use.
  *
  ***************************************************************************/
 
  static char *const _id =
-"$Id: getprinter.c,v 1.27 2002/04/01 17:54:51 papowell Exp $";
+"$Id: getprinter.c,v 1.71 2004/05/03 20:24:02 papowell Exp $";
 
 
 #include "lp.h"
@@ -109,9 +109,9 @@ void Fix_Rm_Rp_info(char *report_conflict, int report_len )
 			*s++ = 0;
 			Set_DYN(&RemoteHost_DYN, s );
 			if( (s = safestrchr(RemoteHost_DYN,'%')) ){
-				*s++ = 0;
-				Set_DYN(&Lpd_port_DYN,s);
+				Set_DYN(&Unix_socket_path_DYN, 0 );
 			}
+			/* force connection via TCP/IP */
 			goto done;
 		}
 		/* we search for the values in the printcap */
@@ -143,6 +143,9 @@ void Fix_Rm_Rp_info(char *report_conflict, int report_len )
 		}
 		if(DEBUGL2)Dump_line_list("Fix_Rm_Rp_info - final PC_entry_line_list",
 			&PC_entry_line_list );
+		Find_default_tags( &PC_entry_line_list, Pc_var_list, "client." );
+		Find_tags( &PC_entry_line_list, &Config_line_list, "client." );
+		Find_tags( &PC_entry_line_list, &PC_entry_line_list, "client." );
 		Set_var_list( Pc_var_list, &PC_entry_line_list);
 		if( RemoteHost_DYN && Lp_device_DYN && report_conflict ){
 			SNPRINTF(report_conflict,report_len)
@@ -168,8 +171,7 @@ void Fix_Rm_Rp_info(char *report_conflict, int report_len )
 				*s++ = 0;
 				Set_DYN(&RemoteHost_DYN, s );
 				if( (s = safestrchr(RemoteHost_DYN,'%')) ){
-					*s++ = 0;
-					Set_DYN(&Lpd_port_DYN,s);
+					Set_DYN(&Unix_socket_path_DYN, 0 );
 				}
 				goto done;
 			}
@@ -188,8 +190,7 @@ void Fix_Rm_Rp_info(char *report_conflict, int report_len )
 				*s++ = 0;
 				Set_DYN(&RemoteHost_DYN, s );
 				if( (s = safestrchr(RemoteHost_DYN+1,'%')) ){
-					*s++ = 0;
-					Set_DYN(&Lpd_port_DYN,s);
+					Set_DYN(&Unix_socket_path_DYN, 0 );
 				}
 			}
 		}
@@ -225,13 +226,16 @@ void Fix_Rm_Rp_info(char *report_conflict, int report_len )
 			s = Queue_name_DYN;
 		}
 		Set_DYN(&Printer_DYN,s);
-
 		DEBUG2("Fix_Rm_Rp_info: found '%s'", Printer_DYN );
 	}
 	if(DEBUGL2)Dump_line_list("Fix_Rm_Rp_info - PC_alias_line_list",
 		&PC_alias_line_list );
 	if(DEBUGL2)Dump_line_list("Fix_Rm_Rp_info - PC_entry_line_list",
 		&PC_entry_line_list );
+	/* now get the Server_xxx variables */
+	Find_default_tags( &PC_entry_line_list, Pc_var_list, "server." );
+	Find_tags( &PC_entry_line_list, &Config_line_list, "server." );
+	Find_tags( &PC_entry_line_list, &PC_entry_line_list, "server." );
 	Set_var_list( Pc_var_list, &PC_entry_line_list);
 	if( RemoteHost_DYN && Lp_device_DYN && report_conflict ){
 		SNPRINTF(report_conflict,report_len)
@@ -245,8 +249,7 @@ void Fix_Rm_Rp_info(char *report_conflict, int report_len )
 		if( *s == 0 ) s = 0;
 		Set_DYN(&RemoteHost_DYN, s );
 		if( (s = safestrchr(RemoteHost_DYN,'%')) ){
-			*s++ = 0;
-			Set_DYN(&Lpd_port_DYN,s);
+			Set_DYN(&Unix_socket_path_DYN, 0 );
 		}
 		Set_DYN(&Lp_device_DYN,0);
 	} else if( Lp_device_DYN ){
@@ -274,6 +277,65 @@ void Fix_Rm_Rp_info(char *report_conflict, int report_len )
  done:
 
 	Expand_vars();
+	/*
+	 * make sure that these entries are in the printcap file
+	 */
+	if( Pc_entries_required_DYN){
+		struct line_list list;
+		struct keywords *var_list = Pc_var_list;
+		int i;
+		Init_line_list( &list );
+		Split(&list,Pc_entries_required_DYN,File_sep,0,0,0,0,0,0);
+		for( i = 0; i < list.count; ++i ){
+			const char *t;
+			int mid;
+			char *s = list.list[i];
+			if( ISNULL(s) ) continue;
+			DEBUG3( "Fix_Rm_Rp_info: checking '%s'", s );
+			if( !Find_first_key( &PC_entry_line_list, s, Hash_value_sep, &mid ) ){
+				t = safestrpbrk(PC_entry_line_list.list[mid], Option_value_sep );
+				DEBUG1( "Fix_Rm_Rp_info: FOUND %s VALUE %s", s, t );
+				continue;
+			}
+			if( !Find_first_key( &Config_line_list, s, Hash_value_sep, &mid ) ){
+				t = safestrpbrk(Config_line_list.list[mid], Hash_value_sep );
+				DEBUG1( "Fix_Rm_Rp_info: CONFIG %s VALUE %s", s, t );
+				Set_str_value(&PC_entry_line_list, s, t);
+				continue;
+			}
+			for( var_list = Pc_var_list; var_list->keyword; ++var_list ){
+				if( !strcmp(var_list->keyword, s) ){
+					int type = var_list->type;				/* type of entry */
+					int v;
+					void *p = var_list->variable;	/* address of variable */
+					if( !(p) ) break;
+					switch(type){
+					case FLAG_K:
+						v =	*(int *)(p);
+						DEBUG1( "Fix_Rm_Rp_info: VAR %s FLAG %d", var_list->keyword, v);
+						Set_flag_value(&PC_entry_line_list, var_list->keyword, v);
+						break;
+					case INTEGER_K:
+						v =	*(int *)(p);
+						DEBUG1( "Fix_Rm_Rp_info: VAR %s INT %d", var_list->keyword, v);
+						Set_decimal_value(&PC_entry_line_list, var_list->keyword, v);
+						break;
+					case STRING_K:
+						t = *(char **)(p);
+						DEBUG1( "Fix_Rm_Rp_info: VAR %s= '%s'", var_list->keyword, t );
+						if( t ){
+							Set_str_value(&PC_entry_line_list, var_list->keyword, t);
+						}
+						break;
+					default:
+						break;
+					}
+					break;
+				}
+			}
+		}
+		Free_line_list( &list );
+	}
 	DEBUG1("Fix_Rm_Rp_info: Printer '%s', Queue '%s', Lp '%s', Rp '%s', Rh '%s'",
 		Printer_DYN, Queue_name_DYN, Lp_device_DYN,
 		RemotePrinter_DYN, RemoteHost_DYN );
@@ -305,7 +367,7 @@ void Get_all_printcap_entries(void)
 			&PC_alias_line_list,
 			&PC_names_line_list, &PC_order_line_list,
 			&PC_info_line_list, 0, 0 )) ){
-		if( !(t = Find_str_value( &PC_entry_line_list, ALL, Value_sep )) ){
+		if( !(t = Find_str_value( &PC_entry_line_list, ALL )) ){
 			t = "all";
 		}
 		DEBUG1("Get_all_printcap_entries: '%s' has '%s'",s,t);
@@ -339,7 +401,7 @@ void Show_formatted_info( void )
 	s = Join_line_list_with_sep(&PC_alias_line_list,"|");
 	if( Write_fd_str( 1, s ) < 0 ) cleanup(0);
 	if(s) free(s); s = 0;
-	Escape_colons( &PC_entry_line_list );
+	/* Escape_colons( &PC_entry_line_list ); */
 	s = Join_line_list_with_sep(&PC_entry_line_list,"\n :");
 	Expand_percent( &s );
 	if( s ){
