@@ -1,14 +1,14 @@
 /***************************************************************************
  * LPRng - An Extended Print Spooler System
  *
- * Copyright 1988-2002, Patrick Powell, San Diego, CA
+ * Copyright 1988-2003, Patrick Powell, San Diego, CA
  *     papowell@lprng.com
  * See LICENSE for conditions of use.
  *
  ***************************************************************************/
 
  static char *const _id =
-"$Id: lpd_control.c,v 1.27 2002/04/01 17:54:52 papowell Exp $";
+"$Id: lpd_control.c,v 1.71 2004/05/03 20:24:02 papowell Exp $";
 
 
 #include "lp.h"
@@ -176,7 +176,7 @@ int Job_control( int *sock, char *input )
 			/* we have Nprinter user move jobid* target */
 			if( tokencount < 5 ){
 				SNPRINTF( error, sizeof(error))
-					_("Use: OP_MOVE printer (user|jobid)* target") );
+					_("Use: move printer (user|jobid)* target") );
 				goto error;
 			}
 			break;
@@ -268,7 +268,7 @@ void Do_queue_control( char *user, int action, int *sock,
 
 	c = Debug;
 	i = DbgFlag;
-	end = Find_str_value(&Spool_control,DEBUG,Value_sep);
+	end = Find_str_value(&Spool_control,DEBUG);
 	if( !end ) end = New_debug_DYN;
 	Parse_debug( end, 0 );
 
@@ -321,6 +321,9 @@ void Do_queue_control( char *user, int action, int *sock,
 	case OP_PRINTCAP:
 		Do_control_printcap( sock );
 		goto done;
+	case OP_PPD:
+		Do_control_ppd( sock );
+		goto done;
 	case OP_STATUS:
 		if( Do_control_status( sock,
 			error, errorlen ) ){
@@ -516,7 +519,7 @@ void Do_queue_control( char *user, int action, int *sock,
 				SNPRINTF(msg,sizeof(msg))_(
 				"WARNING: the main load balance server may have exited before\n"
 				"it could be informed that there were new jobs.\n"
-                "Please use 'lpc start %s' to start the server"),
+                "Please use 'lpc start %s' to start the server\n"),
 					Server_queue_name_DYN );
 				Write_fd_str(*sock,msg);
 			}
@@ -528,7 +531,7 @@ void Do_queue_control( char *user, int action, int *sock,
 	/* start the server if necessary */
 	switch( action ){
 	case OP_KILL:
-			sleep(1);
+			plp_sleep(1);
 			serverpid = 0;
 			break;
 	case OP_START:
@@ -594,7 +597,7 @@ int Do_control_file( int action, int *sock,
 	Init_job(&job);
 	Free_line_list(&Sort_order);
 	if( Scan_queue( &Spool_control, &Sort_order,
-			0,0,0,0,0,0) ){
+			0,0,0,0,0,0,0,0) ){
 		err = errno;
 		SNPRINTF(error, errorlen)
 			"Do_control_file: cannot read '%s' - '%s'",
@@ -613,16 +616,16 @@ int Do_control_file( int action, int *sock,
 		 * check to see if this entry matches any of the patterns
 		 */
 		Free_job(&job);
-		Get_hold_file( &job, Sort_order.list[i] );
+		Get_hold_file( &job, Sort_order.list[i], 0 );
 		DEBUGFC(DCTRL2)Dump_job("Do_control_file - getting info",&job);
-		identifier = Find_str_value(&job.info,IDENTIFIER,Value_sep);
-		if( identifier == 0 ) identifier = Find_str_value(&job.info,TRANSFERNAME,Value_sep);
+		identifier = Find_str_value(&job.info,IDENTIFIER);
+		if( identifier == 0 ) identifier = Find_str_value(&job.info,CFTRANSFERNAME);
 		if( identifier == 0 ) continue;
 		DEBUGF(DCTRL4)("Do_control_file: checking id '%s'", identifier );
 
-		Perm_check.user = Find_str_value(&job.info,LOGNAME,Value_sep);
+		Perm_check.user = Find_str_value(&job.info,LOGNAME);
 		Perm_check.host = 0;
-		s = Find_str_value(&job.info,FROMHOST,Value_sep);
+		s = Find_str_value(&job.info,FROMHOST);
 		if( s && Find_fqdn( &PermHost_IP, s ) ){
 			Perm_check.host = &PermHost_IP;
 		}
@@ -637,7 +640,7 @@ int Do_control_file( int action, int *sock,
 			continue;
 		}
 
-		destinations = Find_flag_value(&job.info,DESTINATIONS,Value_sep);
+		destinations = Find_flag_value(&job.info,DESTINATIONS);
 
 		update_dest = 0;
 
@@ -658,7 +661,7 @@ int Do_control_file( int action, int *sock,
 
 		DEBUGFC(DCTRL4){
 			LOGDEBUG("Do_control_file: selected id '%s'", identifier );
-			s = Find_str_value(&job.destination,IDENTIFIER,Value_sep);
+			s = Find_str_value(&job.destination,IDENTIFIER);
 			LOGDEBUG("Do_control_file: update_dest %d, id '%s'", update_dest, s );
 		}
 		
@@ -685,6 +688,8 @@ int Do_control_file( int action, int *sock,
 			Set_flag_value(&job.info,PRIORITY_TIME,0 );
 			Set_flag_value(&job.info,DONE_TIME,0 );
 			Set_flag_value(&job.info,REMOVE_TIME,0 );
+			Set_flag_value(&job.info,ERROR_TIME,0 );
+			Set_str_value(&job.info,ERROR,0 );
 			setmessage( &job, TRACE, "LPC move" );
 			break;
 		case OP_RELEASE:
@@ -806,7 +811,7 @@ int Do_control_status( int *sock,
 	Free_line_list(&Spool_control);
 	Get_spool_control( Queue_control_file_DYN, &Spool_control );
 	if( Scan_queue( &Spool_control, &Sort_order, &printable,
-			&held, &move,1,&err,&done) ){
+			&held, &move,1,&err,&done,0,0) ){
 		SNPRINTF( error, errorlen)
 			"Do_control_status: cannot read '%s' - '%s'",
 			Spool_dir_DYN, Errormsg(errno) );
@@ -968,7 +973,7 @@ int Do_control_class( int *sock,
 		goto error;
 	}
 
-	s = Find_str_value(&Spool_control,CLASS,Value_sep);
+	s = Find_str_value(&Spool_control,CLASS);
 
 	if( s ){
 		SNPRINTF( forward, sizeof(forward)) _("classes printed '%s'\n"),
@@ -1070,6 +1075,42 @@ int Do_control_printcap( int *sock )
 	return(0);
 }
 
+
+/***************************************************************************
+ * Do_control_ppd:
+ *  get the PPD file
+ * 1. get the ppd file name
+ * 2. if no ppdfile, write nothint
+ ***************************************************************************/
+
+int Do_control_ppd( int *sock )
+{
+	char *file = Ppd_file_DYN;
+	char buffer[LARGEBUFFER];
+	int fd, count;
+	struct stat statb;			/* status of file */
+
+	/* get the spool entries */
+
+	if( !ISNULL(file) ){
+		if( (fd = Checkread( file, &statb )) < 0 ){
+			SNPRINTF(buffer, sizeof(buffer)) "%s: cannot open '%s' - '%s'\n",
+				Printer_DYN, file, Errormsg(errno) );
+			Write_fd_str( *sock, buffer );
+		} else {
+			while( (count = ok_read(fd, buffer, sizeof(buffer)-1)) > 0 ){
+				if( Write_fd_len( *sock, buffer, count ) < 0 ) cleanup(0);
+			}
+			if( count < 0 ){
+				SNPRINTF(buffer, sizeof(buffer)) "%s: error reading '%s' - '%s'\n",
+					Printer_DYN, file, Errormsg(errno) );
+				if( Write_fd_str( *sock, buffer ) < 0 ) cleanup(0);
+			}
+		}
+	}
+	return(0);
+}
+
 int Do_control_defaultq( int *sock )
 {
 	char msg [LINEBUFFER];
